/*
 * (c)Felix Bukatz v0.5.1 21.05.2014
 * 		"Mission SkyDiver"
 * Fast-Paced-SideScrolling Action BallDropper :D
 * Controls: Up/Down/Shoot - Enemy: Up/Down/Move
 * Der Spieler bewegt seinen "Roboter" auf und ab
 * und nimmt den Ball auf oder legt ihn ab. Der Bot macht dasselbe.
 * Spieler werden fr fehler bestraft
 * 
 * TODO:
 * Sounds,
 * Effekte,
 * Optimierung der Performance und stabilitt der HauptKlasse,
 * Code Clean-Up,
 * echtes Ball movement mit States (einlochen und aufnehmen)
 * Double Buffering?
 * Rotation von Objekten?
 */

package Game;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import Scenery.Level;
import Scenery.Loading;
import Scenery.Menu;
import Scenery.Options;

public class BallsDeep {

	//VAR PRIVATE
	private static Window frame;
	
	private static int fps = 60,
					   frameInLastSecond = 0,
					   framesInCurrentSecond = 0,
					   difficulty = 12;
	
	private static boolean isRunning = false;
	private static BufferedImage backBuffer;
	
	private static long time, 
						nextSecond = System.currentTimeMillis() + 1000;;

	
	
	
	//Instanzen fr die setMethode
	private static Menu menu;
	private static Options optionen;
	private static Level level;
	private static InputHandler input;
	
	//VAR PUBLIC
	public static enum GameState {MENU,OPTIONS,LEVEL};
	public static GameState game;
	
	public static void main(String[] args) 
	{
		//Vorbereitung
		initialize();
		System.exit(0);
	}
	//Initialisierungsmethode
	public static void initialize()
	{
		//Initialisiere das Fenster
		frame = new Window();
		input = new InputHandler(frame);
		backBuffer = new BufferedImage(frame.getWidth(), frame.getHeight(), BufferedImage.TYPE_INT_RGB);

		//Nach der gesamten initialisierung -> Programmstart
		changeState(GameState.MENU);
		isRunning = true;
		System.out.println("Initialisierung abgeschlossen!");
		
		run();
	}
	//Programmstart
	public static void run()
	{
		while(isRunning)
		{
			//An dieser stelle noch Unabhngige Updates von der Rendermethode ermglichen
			//damit das spiel in eigenem tempo luft. Dauert das Rendern zu lange wird es geskipped
			//kommt das update zu frh, wird es geskipped (TODO)
			update();
			render();
			limitFPS();
			printFPS();
		}
	}
	//Logikmethode
	public static void update()
	{
		time = System.currentTimeMillis();
		switch(game)
		{
		case MENU:
		{
			menu.update(input);
		}
			break;
		case OPTIONS:
		{
			optionen.update(input);
		}
			break;
		case LEVEL:
		{
			level.update(input);
		}
			break;
		}
	}
	//Rendermethode
	public static void render()
	{
		//Vorbereitung fr Backbuffer und Renderer
		Graphics g =frame.getCanvasGraphics(); 
		Graphics bbg =backBuffer.getGraphics();
		
		//Wenn nichts da ist, weier Hintergrund 
		bbg.setColor(Color.WHITE);
		bbg.fillRect(frame.insets.left, frame.insets.top, 640, 640);
		
		switch(game)
		{
		case MENU:
		{
			menu.render(bbg);
		}
			break;
		case OPTIONS:
		{
			optionen.render(bbg);
		}
			break;
		case LEVEL:
		{
			level.render(bbg);
		}
			break;
		}
		g.drawImage(backBuffer, frame.insets.left, frame.insets.top, 640, 640, frame);
	}
	//Renderkontrolle
	public static void limitFPS()
	{
		
		time = (1000 / fps) - (System.currentTimeMillis() - time); 
         
         if (time > 0) 
         { 
                 try 
                 { 
                         Thread.sleep(time); 
                 } 
                 catch(Exception e)
                 {
                 	
                 } 
         } 
		
	}
	
	public static void printFPS()
	{
		 long currentTime = System.currentTimeMillis();
	        if (currentTime > nextSecond) {
	            nextSecond += 1000;
	            frameInLastSecond = framesInCurrentSecond;
	            framesInCurrentSecond = 0;
	        }
	        framesInCurrentSecond++;
	        System.out.println("Aktuelle FPS: "+Long.toString(frameInLastSecond)+" Maximum FPS: "+ Integer.toString(fps));
	}
	
	public static void changeState(GameState newState)
	{
		game = newState;
		switch(game)
		{
		case MENU:
		{
			Loading menuLaden = new Loading(frame.getGraphics());
			menu = new Menu();
		}
			break;
		case OPTIONS:
		{
			Loading optionenLaden = new Loading(frame.getGraphics());
			optionen = new Options();
		}
			break;
		case LEVEL:
		{
			Loading levelLaden = new Loading(frame.getGraphics());
			level = new Level(difficulty);
		}
			break;
		}
	}

	public static void endGame()
	{
		isRunning = false;
	}
	public static void setDifficulty(int newDif)
	{
		difficulty += newDif;
		
		if(difficulty > 25)
		{
			difficulty = 25;
		}
		if(difficulty < 1)
		{
			difficulty = 1;
		}
		
	}
	public static int getDifficulty()
	{
		return difficulty;
	}
}
