package Scenery;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.RowFilter.ComparisonType;

import Game.BallsDeep;
import Game.BallsDeep.GameState;
import Game.InputHandler;
import Objects.Ball;
import Objects.COM;
import Objects.ComHole;
import Objects.Generator;
import Objects.Hole;
import Objects.Player;

public class Level extends BlackBoard{

	private enum LevelState {PLAY, LOST, WON};
	
	private LevelState state = LevelState.PLAY;
	
	private Hole[] holes = new Hole[5];
	
	private ComHole[] comHoles = new ComHole[5];
	
	private Image[] ballImages = new Image[5];
	
	private ArrayList<Ball> playerBalls = new ArrayList();
	
	private ArrayList<Ball> comBalls = new ArrayList();
	
	private Player player;
	
	private COM com;
	
	private Generator generator, 
					  generatorCom;
	
	private boolean ballInGamePlayer = true, 
					ballInGameCom = false, 
					ballWaiting = false, 
					ballComWaiting = false, 
					coolDownPlayer = false, 
					coolDownPlayerMinus = false, 
					coolDownCom = false;
	
	private int playerScore, 
				comScore, 
				timer = 10, 
				coolDownPlayerSec = 100, 
				coolDownPlayerMinusSec = 100 , 
				coolDownComSec = 100, 
				difficulty;
	
	private long lastSystemTime = System.currentTimeMillis(),
				 currentSystemTime;
	
	private Image winImage,
				  lostImage,
				  scoreBack,
				  timeBack;
	
	public Level(int difficulty)
	{
		try
		{
			this.background = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("menu_back.png"));
			this.winImage = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("gewonnen.png"));
			this.lostImage = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("verloren.png"));
			this.scoreBack = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("score_back.png"));
			this.timeBack = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("time_back.png"));
			//Ball Bilder Laden:
			for(int i = 0; i < ballImages.length; i++)
			{
				ballImages[i] = ImageIO.read(this.getClass().getClassLoader().getResourceAsStream("ball_"+Integer.toString(i)+".png"));
			}
		}
		catch(Exception e)
		{
			System.out.println("Level Ladevorgang fehlgeschlagen!");
			System.exit(0);
		}
		
		//Schwierigkeit bergeben
		this.difficulty = difficulty;
		coolDownComSec -= this.difficulty*3;
		
		//Erstellen von LevelObjekten
		player = new Player();
		com = new COM();
		generator = new Generator("player");
		generatorCom = new Generator("com");
		
		createBall();
		createComBall();
		
		for(int i = 0; i < holes.length;i++)
		{
			holes[i] = new Hole(i);
			comHoles[i] = new ComHole(i);
		}
			
	}
	@Override
	public void update(InputHandler input) {
		
		//Status
		switch(state)
		{
		case PLAY:
			//CoolDownRechnung
			coolDown();
			//Neuer Ball
			newBall();
			//Akteure Updaten
			updatePlayers(input);
			//Objekte Updaten
			updateObjects(input);
			//Timer ablaufen lassen:
			timer();
			//Frame beenden durch drcken von Escape
			if(input.isKeyDown(KeyEvent.VK_ESCAPE))
			{
				BallsDeep.changeState(GameState.MENU);
			}
			break;
		case LOST:
			//Nichts tun auer den Timer abwarten bis man ins Hauptmenu gelangt
			currentSystemTime = System.currentTimeMillis();
			
			if(currentSystemTime - lastSystemTime >= 1000)
			{
				timer -= 1;
				lastSystemTime = currentSystemTime;
			}
			if(timer <= 0)
			{
				gameEnd();
			}
			if(timer <= 0)
			{
				BallsDeep.changeState(BallsDeep.GameState.MENU);
			}
			break;
		case WON:
			//Nichts tun auer den Timer abwarten bis man ins Hauptmenu gelangt
			currentSystemTime = System.currentTimeMillis();
			
			if(currentSystemTime - lastSystemTime >= 1000)
			{
				timer -= 1;
				lastSystemTime = currentSystemTime;
			}
			if(timer <= 0)
			{
				gameEnd();
			}
			if(timer <= 0)
			{
				BallsDeep.changeState(BallsDeep.GameState.MENU);
			}
			break;
			
			
		}
		
		
	}

	@Override
	public void render(Graphics g) {

		g.drawImage(background,0,-25,null);
		
		for(int i = 0; i < holes.length;i++)
		{
			holes[i].render(g);
		}
		
		for(int i = 0; i < comHoles.length;i++)
		{
			comHoles[i].render(g);
		}
		
		generator.render(g);
		generatorCom.render(g);
		player.render(g);
		com.render(g);
		
		for(int i = 0; i < playerBalls.size();i++)
		{
			playerBalls.get(i).render(g);
		}
		
		for(int i = 0; i < comBalls.size();i++)
		{
			comBalls.get(i).render(g);
		}
		//Score und time anzeigen auf einem Hintergrund
		g.setFont(new Font("Impact", Font.PLAIN, 36));
		g.drawImage(scoreBack,10,15,null);
		g.drawString(Integer.toString(playerScore), 34, 55);
		g.drawImage(scoreBack,550,15,null);
		g.drawString(Integer.toString(comScore), 580, 55);
		g.drawImage(timeBack,285,15,null);
		g.drawString(Integer.toString(timer),305, 55);
		
		
		
		//Status
		switch(state)
		{
		case LOST:
			g.drawImage(lostImage,0,0,null);
			break;
		case WON:
			g.drawImage(winImage,0,0,null);
			break;
		}
	}
	void timer()
	{
		long currentSystemTime = System.currentTimeMillis();
		
		if(currentSystemTime - lastSystemTime >= 1000)
		{
			timer -= 1;
			lastSystemTime = currentSystemTime;
		}
		if(timer <= 0)
		{
			timer = 5;
			gameEnd();
		}
	}
	@Override
	void changeGameState() {
		// das hier ist garnicht mehr notwendig
		
	}
	public void createBall()
	{
		playerBalls.add(new Ball(ballImages, generator.getPosX(), generator.getPosY()));
		ballWaiting = true;
	}
	public void createComBall()
	{
		comBalls.add(new Ball(ballImages, generatorCom.getPosX(), generatorCom.getPosY()));
		ballComWaiting = true;
	}
	public void coolDown()
	{
		//Spieler STUN bei fehler CoolDown
		if(coolDownPlayer)
		{
			coolDownPlayerSec -=1;
		}
		if(coolDownPlayerSec == 0)
		{
			coolDownPlayer = false;
			coolDownPlayerSec = 100;
		}
		//Spieler Punktabzugs Cooldown fr Menschliche Geschwindigkeit
		if(coolDownPlayerMinus)
		{
			coolDownPlayerMinusSec -=1;
		}
		if(coolDownPlayerMinusSec == 0)
		{
			coolDownPlayerMinus = false;
			coolDownPlayerMinusSec = 100;
		}
		//COM STUN bei fehler CoolDown
		if(coolDownCom)
		{
			coolDownComSec -=1;
		}
		if(coolDownComSec == 0)
		{
			coolDownCom = false;
			coolDownComSec = 100-this.difficulty*3;
		}
		
	}
	private void checkComAction()
	{
		//Dient der Manipulation der Com Aktionen, je nach Situation im Level
		//Es muss geprft werden, ob er am generator angekommen ist um einen Ball zu holen,
		//ob das aktuelle loch das richtige fr ihn ist, dann werden die movements gendert und er macht sein zeug.
		
		//Nach dem Absetzen:
		if(com.getAction() == COM.ComAction.DEPLOY)
		{
			com.setAction(COM.ComAction.DOWN);
		}
		//Am generator angekommen:
		if(com.getHotSpotY() >= generatorCom.getHotSpotY())
		{
			com.setAction(COM.ComAction.GET);
		}
		//Ball aufnehmen
		if(ballComWaiting && com.getAction() == COM.ComAction.GET)
		{
			for(int i = 0; i < comBalls.size(); i++)
			{
				comBalls.get(i).toggleComHook();
			}
			
			ballComWaiting = false;
			ballInGameCom = true;
			
			com.setAction(COM.ComAction.UP);
		}
		//Dauernd prfen ob er an einem Loch ist
		for(int i = 0; i < comHoles.length; i++)
		{
			if(com.getHotSpotY() <= comHoles[i].getHotSpotY()+3 && com.getHotSpotY() >= comHoles[i].getHotSpotY()-3 && !ballComWaiting)
			{
				for(int n = 0; n < comBalls.size(); n++)
				{
					//Was passiert wenns passt?
					if(comBalls.get(n).getType().equals(comHoles[i].getType()))
						{
							comBalls.clear();
							comScore++;
							ballInGameCom = false;
							com.setAction(COM.ComAction.DEPLOY);
						} //Was passiert wenn nicht?
					else  
						{
						//Spieler bekommt Cooldown und Punkte strafe. COM nur cooldown, denn der ist eh dumm und wei es nicht besser
							coolDownCom = true;								
											
						}
				}
			}
		}
		
	}
	void gameEnd()
	{
		if(playerScore > comScore)
		{
			state = LevelState.WON;
		}
		else if(comScore > playerScore)
		{
			state = LevelState.LOST;
		}
		else if(comScore == playerScore)
		{
			//Gleichstand??
			state = LevelState.WON;
		}
	}
	void newBall()
	{
		if(!ballWaiting && !ballInGamePlayer)
		{
			createBall();
		}
		if(!ballComWaiting && !ballInGameCom)
		{
			createComBall();
		}
	}
	void updatePlayers(InputHandler input)
	{
		if(!coolDownPlayer)
		{
			player.update(input);
		}
		if(!coolDownCom)
		{
			com.update(input);
		}
	}
	void updateObjects(InputHandler input)
	{
generator.update(input);
		
		for(int i = 0; i < playerBalls.size();i++)
		{
			playerBalls.get(i).update(input, player.getHotSpotX(), player.getHotSpotY(), com.getHotSpotX(), com.getHotSpotY());
		}
		
		for(int i = 0; i < comBalls.size();i++)
		{
			comBalls.get(i).update(input,player.getHotSpotX(), player.getHotSpotY(), com.getHotSpotX(), com.getHotSpotY());
		}
		
		for(int i = 0; i < holes.length;i++)
		{
			holes[i].update(input);
		}
		
		for(int i = 0; i < comHoles.length;i++)
		{
			comHoles[i].update(input);
		}
		
		//Prfen ob der Spieler am Generator steht und ob er die Leertaste zum aufnehmen drckt, falls ein ball verfgbar ist
		
		if(ballWaiting && player.getHotSpotY() <= generator.getHotSpotY()+ 10  && player.getHotSpotY() >= generator.getHotSpotY()- 10 && input.isKeyDown(KeyEvent.VK_SPACE))
		{
			for(int i = 0; i < playerBalls.size(); i++)
			{
				playerBalls.get(i).togglePlayerHook();
			}
			
			ballWaiting = false;
			ballInGamePlayer = true;
		}
		//Prfen ob er einen Ball hat und vor dem richtigen loch steht wenn er die leertaste drckt
		
		if(ballInGamePlayer && input.isKeyDown(KeyEvent.VK_SPACE))
		{
			for(int i = 0; i < holes.length; i++)
			{
				if(player.getHotSpotY() <= holes[i].getHotSpotY()+10 && player.getHotSpotY() >= holes[i].getHotSpotY()-10)
				{
					for(int n = 0; n < playerBalls.size(); n++)
					{
						//Was passiert wenns passt?
						if(playerBalls.get(n).getType().equals(holes[i].getType()))
							{
								playerBalls.clear();
								playerScore++;
								ballInGamePlayer = false;
								
							} //Was passiert wenn nicht?
						else
							{
							//Spieler bekommt Cooldown und Punkte strafe. COM nicht, denn der ist eh dumm und wei es nicht besser
								coolDownPlayer = true;								
								if(!coolDownPlayerMinus)
								{
									playerScore -= 1;
									coolDownPlayerMinus = true;
								}
								
							}
					}
				}
			}
		}
		//Wenn der Rest erledigt ist, check den Com
		checkComAction();
	}
}
